require 'faraday'
require 'google_plus/request/gateway'
require 'google_plus/request/multipart_with_file'
require 'google_plus/request/phoenix'
require 'google_plus/request/oauth'
require 'google_plus/response/parse_json'
require 'google_plus/response/raise_client_error'
require 'google_plus/response/raise_server_error'

module GooglePlus
  module Connection
  private

    # Returns a Faraday::Connection object
    #
    # @return [Faraday::Connection]
    def connection(options={})
      merged_options = connection_options.merge({
        :headers => {
          :accept => 'application/json',
          :user_agent => user_agent,
        },
        :proxy => proxy,
        :ssl => {:verify => false},
        :url => options.fetch(:endpoint, api_endpoint),
      })
    
     Faraday.new(merged_options) do |builder|
        builder.use GooglePlus::Request::Phoenix if options[:phoenix]
        builder.use GooglePlus::Request::MultipartWithFile
        builder.use GooglePlus::Request::GooglePlusOAuth, credentials if credentials?
        builder.use Faraday::Request::Multipart
        builder.use Faraday::Request::UrlEncoded
        builder.use GooglePlus::Request::Gateway, gateway if gateway
        builder.use GooglePlus::Response::RaiseClientError
        builder.use GooglePlus::Response::ParseJson unless options[:raw]
        builder.use GooglePlus::Response::RaiseServerError
        builder.adapter(adapter)
     end
    end

  end
end

