require 'faraday'
require 'google_plus/error/bad_request'
require 'google_plus/error/enhance_your_calm'
require 'google_plus/error/forbidden'
require 'google_plus/error/not_acceptable'
require 'google_plus/error/not_found'
require 'google_plus/error/unauthorized'

module GooglePlus
  module Response
    class RaiseClientError < Faraday::Response::Middleware

      def on_complete(env)
        case env[:status].to_i
        when 400
          raise GooglePlus::Error::BadRequest.new(error_message(env), env[:response_headers])
        when 401
          raise GooglePlus::Error::Unauthorized.new(error_message(env), env[:response_headers])
        when 403
          raise GooglePlus::Error::Forbidden.new(error_message(env), env[:response_headers])
        when 404
          raise GooglePlus::Error::NotFound.new(error_message(env), env[:response_headers])
        when 406
          raise GooglePlus::Error::NotAcceptable.new(error_message(env), env[:response_headers])
        when 420
          raise GooglePlus::Error::EnhanceYourCalm.new(error_message(env), env[:response_headers])
        end
      end

    private

      def error_message(env)
        "#{env[:method].to_s.upcase} #{env[:url].to_s}: #{env[:status]}#{error_body(env[:body])}"
      end

      def error_body(body)
        if body.nil?
          nil
        elsif body['error']
          ": #{body['error']}"
        elsif body['errors']
          first = Array(body['errors']).first
          if first.kind_of? Hash
            ": #{first['message'].chomp}"
          else
            ": #{first.chomp}"
          end
        end
      end

    end
  end
end

